//
//  DIContainer.swift
//  DontWaste
//
//  Created by David Aldorf on 26.02.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import Swinject

public class DIContainer {
    
    static let shared = DIContainer()
    
    var container = Container()
    
    init() {
        
        createServiceDependencies()
        createViewModelDependencies()
        createViewControllerDependencies()
        Container.loggingFunction = nil
    }
}

public extension DIContainer {
    
    // Registrace Modelové vrstvy aplikace
    func createServiceDependencies() {
        
        container.register(FacebookLoginService.self) { _ in
            FacebookLoginService()
        }
        container.register(FirebaseAuthService.self) {
            _ in FirebaseAuthService()
        }
        container.register(FirebaseService.self) {
            _ in FirebaseService()
        }
    }
}

public extension DIContainer {
    
    // Registrace ViewModel vrstvy aplikace
    func createViewModelDependencies() {
        
        container.register(LoginVM.self) { resolver in
            return LoginVM(
                facebookLoginService: resolver.resolve(FacebookLoginService.self)!,
                firebaseAuthService: resolver.resolve(FirebaseAuthService.self)!
            )
        }
        container.register(FridgeListVM.self) {
            resolver in FridgeListVM(firebaseService: resolver.resolve(FirebaseService.self)!)
        }
        container.register(ScannerVM.self) {
            resolver in ScannerVM(firebaseService: resolver.resolve(FirebaseService.self)!)
        }
        container.register(RecipeListVM.self) {
            resolver in RecipeListVM(firebaseService: resolver.resolve(FirebaseService.self)!)
        }
    }
}

public extension DIContainer {
    
    struct StoryboardName {
        
        static let login = "Login"
        static let fridge = "Fridge"
        static let main = "MainTabBar"
        static let recipes = "Recipe"
    }
    
    // Registrace View vrstvy aplikace
    func createViewControllerDependencies() {
        
        container.register(LoginVC.self) { resolver in
            
            return LoginVC.storyboardInit(StoryboardName.login, viewModel: resolver.resolve(LoginVM.self))
        }
        container.register(FridgeListVC.self) { resolver in
            
            return FridgeListVC.storyboardInit(StoryboardName.fridge, viewModel: resolver.resolve(FridgeListVM.self))
        }
        container.register(ScannerVC.self) { resolver in
            
            return ScannerVC.storyboardInit(StoryboardName.fridge, viewModel: resolver.resolve(ScannerVM.self))
        }
        container.register(MainTabBarController.self) { resolver in
            
            return MainTabBarController.storyboardInit(StoryboardName.main, viewModel: nil)
        }
        container.register(RecipeListVC.self) { resolver in
            
            return RecipeListVC.storyboardInit(StoryboardName.recipes, viewModel: resolver.resolve(RecipeListVM.self))
        }
    }
}

