//
//  Product.swift
//  DontWaste
//
//  Created by David Aldorf on 03.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import Foundation
import ObjectMapper

enum MeasurementUnit: String {
    
    static var count: Int { return MeasurementUnit.miliLitres.hashValue + 1 }
    
    static let datasource: [String] = [
        "ks","g","ml"
    ]
    
    case pieces = "ks"
    case grams = "g"
    case miliLitres = "ml"
}

enum Category: String {
    
    static var count: Int { return Category.sausages.hashValue + 1 }
    
    static let datasource: [String] = [
        "Alkoholické nápoje",
        "Nealkoholické nápoje, káva, čaj a voda",
        "Cereálie, výrobky z obilovin",
        "Dorty, trvanlivé pečivo, jemmé pečivo a sušenky",
        "Vejce a výrobky z vajec",
        "Ovoce",
        "Zelenina",
        "Hlízy, houby, brambory, škrobnaté plodiny",
        "Přísady",
        "Dietetické potraviny",
        "Koktejly, gainery, sportovní doplňky",
        "Luštěniny ořechy, oleje a další semena",
        "Mléčné výrobky, sýry (mimo másla)",
        "Drůbež, zvěřina, pernatá zvěř, droby, ostatní maso",
        "Oleje, tuky, máslo",
        "Pečivo, chléb a housky",
        "Maso",
        "Ryby a rybí výrobky, plody moře atd.",
        "Cukr a cukrovinky čokoládové, zmrzlina",
        "Těstoviny",
        "Uzeniny, párky, klobásy, salámy, masné výrobky"
    ]
    
    case alco = "Alkoholické nápoje"
    case nealco = "Nealkoholické nápoje, káva, čaj a voda"
    case cereals = "Cereálie, výrobky z obilovin"
    case sweetPastry = "Dorty, trvanlivé pečivo, jemmé pečivo a sušenky"
    case eggs = "Vejce a výrobky z vajec"
    case fruits = "Ovoce"
    case vegetables = "Zelenina"
    case potatoesMushrooms = "Hlízy, houby, brambory, škrobnaté plodiny"
    case incredients = "Přísady"
    case dieteticFoods = "Dietetické potraviny"
    case foodSuplements = "Koktejly, gainery, sportovní doplňky"
    case luteinNuts = "Luštěniny ořechy, oleje a další semena"
    case cheeseMilk = "Mléčné výrobky, sýry (mimo másla)"
    case poultry = "Drůbež, zvěřina, pernatá zvěř, droby, ostatní maso"
    case oils = "Oleje, tuky, máslo"
    case pastry = "Pečivo, chléb a housky"
    case meat = "Maso"
    case fishes = "Ryby a rybí výrobky, plody moře atd."
    case sweets = "Cukr a cukrovinky čokoládové, zmrzlina"
    case pasta = "Těstoviny"
    case sausages = "Uzeniny, párky, klobásy, salámy, masné výrobky"
}

class Product: Mappable {
    
    var id: String?
    var name: String?
    var expirationDateFrom: Date?
    var expirationDateTo: Date?
    var barCode: Int?
    var quantity: Int?
    var measurementUnitString: MeasurementUnit?
    var categoryString: Category?
    
    init(
        name: String,
        expirationDateFrom: Date,
        expirationDateTo: Date,
        barCode: Int,
        quantity: Int,
        measurementUnitString: MeasurementUnit,
        categoryString: Category
    ) {
        
        self.name = name
        self.expirationDateFrom = expirationDateFrom
        self.expirationDateTo = expirationDateTo
        self.barCode = barCode
        self.quantity = quantity
        self.measurementUnitString = measurementUnitString
        self.categoryString = categoryString
    }
    
    required init?(map: Map) {}
    
    func mapping(map: Map) {
        
        id <- map[Product.firebaseIdKey]
        name <- map["name"]
        expirationDateFrom <- (map["expirationDateFrom"], DateTransform())
        expirationDateTo <- (map["expirationDateTo"], DateTransform())
        barCode <- map["barCode"]
        quantity <- map["quantity"]
        measurementUnitString <- (map["measurementUnit"], EnumTransform<MeasurementUnit>())
        categoryString <- (map["category"], EnumTransform<Category>())
    }
    
    static func empty(data: [String: String] = [:]) -> Product {
        let json = "{}"
        let jsonDictionary = Mapper<Product>.parseJSONStringIntoDictionary(JSONString: json)
        let mapData: Map = Map(mappingType: .fromJSON, JSON: jsonDictionary!, toObject: true)
        return Product(map: mapData)!
    }
}
