//
//  ObjectMapper.swift
//  DontWaste
//
//  Created by David Aldorf on 06.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import ObjectMapper
import FirebaseDatabase

extension Mapper {
    func mapArray(snapshot: DataSnapshot) -> [N] {
        return snapshot.children.map { child -> N? in
            if let childSnap = child as? DataSnapshot {
                return N(snapshot: childSnap)
            }
            return nil
            }.flatMap { $0 }
    }
    
    func map(snapshot: DataSnapshot) -> N? {
        return N(snapshot: snapshot)
    }
}

extension BaseMappable {
    static var firebaseIdKey : String {
        get {
            return "FirebaseIdKey"
        }
    }
    
    init?(snapshot: DataSnapshot) {
        guard var json = snapshot.value as? [String: Any] else {
            return nil
        }
        json[Self.firebaseIdKey] = snapshot.key as Any
        
        self.init(JSON: json)
    }
}
