//
//  FridgeListVM.swift
//  DontWaste
//
//  Created by David Aldorf on 26.02.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import RxSwift
import RxCocoa

class FridgeListVM: ViewModelType {
    
    private let firebaseService: FirebaseService
    
    init(firebaseService: FirebaseService){
        
        self.firebaseService = firebaseService
    }
    
    var navigator: Navigator?
    
    struct Input {
        let trigger: Driver<Void>
        let createPostTrigger: Driver<Void>
        let selection: Driver<IndexPath>
    }
    struct Output {
        let fetching: Driver<Bool>
        let products: Driver<[Product]>
        let createPost: Driver<Void>
        let selectedPost: Driver<Product>
        let error: Driver<Error>
    }
    
    func transform(input: Input) -> Output {
        let activityIndicator = ActivityIndicator()
        let errorTracker = ErrorTracker()
        let products = input.trigger.flatMapLatest {
            return self.firebaseService.getProducts()
                .trackActivity(activityIndicator)
                .debug()
                .trackError(errorTracker)
                .asDriverOnErrorJustComplete()
        }
        
        let fetching = activityIndicator.asDriver()
        let errors = errorTracker.asDriver()
        let selectedPost = input.selection
            .withLatestFrom(products) { (indexPath, products) -> Product in
                return products[indexPath.row]
            }
            .do(onNext: navigator?.toProduct)
        let createPost = input.createPostTrigger
            .do(onNext: navigator?.toCreateProducts)
        
        return Output(fetching: fetching,
                      products: products,
                      createPost: createPost,
                      selectedPost: selectedPost,
                      error: errors)
    }
}
