//
//  LoginVC.swift
//  DontWaste
//
//  Created by David Aldorf on 24.02.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import UIKit
import RxSwift
import RxCocoa

class LoginVC: UIViewController, RxViewControllerProtocol, StoryboardInit {
    
    internal var disposeBag = DisposeBag()
        
    var viewModel: LoginVM?

    @IBOutlet weak var emailTextField: UITextField!
    @IBOutlet weak var passwordTextField: UITextField!
    @IBOutlet weak var facebookLoginButton: RoundedButton!
    @IBOutlet weak var loginButton: RoundedButton!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        bindToVM()
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        
        return .lightContent
    }
    
    internal func bindToVM() {
        
        let input = LoginVM.Input(
            facebookLoginIn: facebookLoginButton.rx.tap.asDriver(),
            loginWithEmail: loginButton.rx.tap.asDriver(),
            email: emailTextField.rx.text.orEmpty.asDriver(),
            password: passwordTextField.rx.text.orEmpty.asDriver()
        )
        
        let output = viewModel?.transform(input: input)
        
        [
            output?.facebookLoginOut.drive(),
            output?.loginWithEmail.drive(),
            output?.error.drive(errorBinding),
            output?.isEmailValid.drive(validationBinder),
            output?.isPasswordValid.drive(validationBinder)
        ].forEach({ $0?.disposed(by: disposeBag) })
    }
    
    var errorBinding: Binder<Error> {
        
        return Binder(self) { error, _  in
            
            let alert = UIAlertController(title: "Save Error",
                                          message: "Something went wrong",
                                          preferredStyle: .alert)
            let action = UIAlertAction(title: "Dismiss",
                                       style: UIAlertActionStyle.cancel,
                                       handler: nil)
            alert.addAction(action)
            
            self.present(alert, animated: true, completion: nil)
        }
    }
    
    var validationBinder: Binder<Bool> {
        
        return Binder(self) { _, _  in
            
            if
                self.emailTextField.text?.isEmailValid == true,
                self.passwordTextField.text?.isPasswordValid == true
            {
                
                self.loginButton.isEnabled = true
            } else {
                
                self.loginButton.isEnabled = false
            }
        }
    }
}
