//
//  ViewController.swift
//  DontWaste
//
//  Created by David Aldorf on 03.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import UIKit

class MainTabBarController: UITabBarController, StoryboardInit {
    
    var viewModel: MainTabBarController? = nil
    
    typealias ViewModel = MainTabBarController
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupUI()
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        
        return .lightContent
    }

    private func setupUI() {
        
        UIApplication.shared.statusBarView?.backgroundColor = UIColor(with: "0096FF")
        
        // Tab Bar Appearance
        
        tabBar.itemPositioning = .centered
        
        tabBar.barTintColor = .white
        tabBar.clipsToBounds = true
        
        tabBar.layer.borderWidth = 0
        tabBar.layer.borderColor = UIColor.white.cgColor
        
        // Tab Bar Contents

        let textAttributes: [NSAttributedStringKey: Any] = [
            NSAttributedStringKey.font: UIFont.systemFont(ofSize: 11),
            NSAttributedStringKey.foregroundColor: UIColor.lightGray
        ]

        let selectedTextAttributes: [NSAttributedStringKey: Any] = [
            NSAttributedStringKey.font: UIFont.systemFont(ofSize: 11),
            NSAttributedStringKey.foregroundColor: UIColor(with: "0096FF")
        ]

        let fridgeListVC = DIContainer.shared.container.resolve(FridgeListVC.self)!

        fridgeListVC.tabBarItem = UITabBarItem(
            title: "Lednice",
            image: #imageLiteral(resourceName: "fridge-tabbar"),
            selectedImage: #imageLiteral(resourceName: "fridge-tabbar")
        )
        fridgeListVC.tabBarItem.setTitleTextAttributes(textAttributes, for: .normal)
        fridgeListVC.tabBarItem.setTitleTextAttributes(selectedTextAttributes, for: .highlighted)
        
        let navigationFridgeVC = UINavigationController(rootViewController: fridgeListVC)
        
        navigationFridgeVC.navigationBar.setBackgroundImage(UIImage(), for: .default)
        navigationFridgeVC.navigationBar.shadowImage = UIImage()
        navigationFridgeVC.navigationBar.backgroundColor = UIColor(with: "0096FF")
        navigationFridgeVC.navigationBar.tintColor = .white
        
        let recipeListVC = DIContainer.shared.container.resolve(RecipeListVC.self)!
        
        recipeListVC.tabBarItem = UITabBarItem(
            title: "Recepty",
            image: #imageLiteral(resourceName: "recipes-tabbar"),
            selectedImage: #imageLiteral(resourceName: "recipes-tabbar")
        )
        recipeListVC.tabBarItem.setTitleTextAttributes(textAttributes, for: .normal)
        recipeListVC.tabBarItem.setTitleTextAttributes(selectedTextAttributes, for: .highlighted)
        
        let navigationRecipesVC = UINavigationController(rootViewController: recipeListVC)
        
        navigationRecipesVC.navigationBar.setBackgroundImage(UIImage(), for: .default)
        navigationRecipesVC.navigationBar.shadowImage = UIImage()
        navigationRecipesVC.navigationBar.backgroundColor = UIColor(with: "0096FF")
        navigationRecipesVC.navigationBar.tintColor = .white

        viewControllers = [
            navigationFridgeVC,
            navigationRecipesVC
        ]
    }
}
