//
//  RecipeDetailVC.swift
//  DontWaste
//
//  Created by David Aldorf on 24.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import UIKit
import RxCocoa
import RxSwift

class RecipeDetailVC: UIViewController, RxViewControllerProtocol, StoryboardInit {
    
    internal var disposeBag = DisposeBag()
    
    var viewModel: RecipeDetailVM?
    
    @IBOutlet weak var productsTableView: UITableView!
    @IBOutlet weak var imageView: UIImageView!
    @IBOutlet weak var nameTextField: UITextField!
    @IBOutlet weak var addProductButton: UIButton!
    @IBOutlet weak var preparationDurationTextField: UITextField!
    @IBOutlet weak var descriptionTextView: UITextView!
    
    override func viewDidLoad() {
        
        super.viewDidLoad()
        
        configureTableView()
        bindToVM()
    }
    
    private func configureTableView() {
        
        productsTableView.delegate = nil
        productsTableView.dataSource = nil
        
        productsTableView.tableFooterView = UIView()
        
        productsTableView.register(RecipeItemCell.self)
        
        productsTableView.refreshControl = UIRefreshControl()
        productsTableView.estimatedRowHeight = 64
        productsTableView.rowHeight = UITableViewAutomaticDimension
    }
    
    internal func bindToVM() {
        
        viewModel?.navigator = Navigator(recipesNavigatorController: navigationController!)
        
        let viewWillAppear = rx.sentMessage(#selector(UIViewController.viewWillAppear(_:)))
            .mapToVoid()
            .asDriverOnErrorJustComplete()
        let pull = productsTableView.refreshControl!.rx
            .controlEvent(.valueChanged)
            .asDriver()
        
        let input = RecipeDetailVM.Input(trigger: Driver.merge(viewWillAppear, pull),
                                       createPostTrigger: (navigationItem.rightBarButtonItem?.rx.tap.asDriver())!,
                                       selection: productsTableView.rx.itemSelected.asDriver())
        let output = viewModel?.transform(input: input)
        //Bind Posts to UITableView
        output?.recipes.drive(
            productsTableView.rx.items(cellIdentifier: "RecipeItemCell", cellType: RecipeItemCell.self)
        ) { (index, recipe: Recipe, cell) in
            
            cell.setup(recipe)
            }.disposed(by: disposeBag)
        //Connect Create Post to UI
        
        output?.fetching
            .drive(productsTableView.refreshControl!.rx.isRefreshing)
            .disposed(by: disposeBag)
        output?.createPost
            .drive()
            .disposed(by: disposeBag)
        output?.selectedPost
            .drive()
            .disposed(by: disposeBag)
    }
}
