//
//  RecipeDetailVM.swift
//  DontWaste
//
//  Created by David Aldorf on 24.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import RxSwift
import RxCocoa

class RecipeDetailVM: ViewModelType {
    
    private let firebaseService: FirebaseService
    
    init(firebaseService: FirebaseService){
        
        self.firebaseService = firebaseService
    }
    
    var navigator: Navigator?
    
    struct Input {
        let trigger: Driver<Void>
        let createPostTrigger: Driver<Void>
        let selection: Driver<IndexPath>
    }
    struct Output {
        let fetching: Driver<Bool>
        let recipes: Driver<[Recipe]>
        let createPost: Driver<Void>
        let selectedPost: Driver<Recipe>
        let error: Driver<Error>
    }
    
    func transform(input: Input) -> Output {
        let activityIndicator = ActivityIndicator()
        let errorTracker = ErrorTracker()
        let recipes = input.trigger.flatMapLatest {
            return self.firebaseService.getRecipes()
                .trackActivity(activityIndicator)
                .debug()
                .trackError(errorTracker)
                .asDriverOnErrorJustComplete()
        }
        
        let fetching = activityIndicator.asDriver()
        let errors = errorTracker.asDriver()
        let selectedPost = input.selection
            .withLatestFrom(recipes) { (indexPath, recipes) -> Recipe in
                return recipes[indexPath.row]
            }
            .do(onNext: navigator?.toRecipe)
        let createPost = input.createPostTrigger
            .do(onNext: navigator?.toCreateProducts)
        
        return Output(fetching: fetching,
                      recipes: recipes,
                      createPost: createPost,
                      selectedPost: selectedPost,
                      error: errors)
    }
}
