//
//  RecipeListVC.swift
//  DontWaste
//
//  Created by David Aldorf on 11.03.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import UIKit
import RxCocoa
import RxSwift

class RecipeListVC: UIViewController, RxViewControllerProtocol, StoryboardInit {
    
    internal var disposeBag = DisposeBag()
    
    var viewModel: RecipeListVM?
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        
        super.viewDidLoad()
        
        navigationItem.rightBarButtonItem = UIBarButtonItem(
            title: "+",
            style: .plain,
            target: self,
            action: nil
        )
        
        configureTableView()
        bindToVM()
    }
    
    private func configureTableView() {
        
        tableView.delegate = nil
        tableView.dataSource = nil
        
        tableView.tableFooterView = UIView()
        
        tableView.register(RecipeItemCell.self)
        
        tableView.refreshControl = UIRefreshControl()
        tableView.estimatedRowHeight = 64
        tableView.rowHeight = UITableViewAutomaticDimension
    }
    
    internal func bindToVM() {
        
        viewModel?.navigator = Navigator(recipesNavigatorController: navigationController!)
        
        let viewWillAppear = rx.sentMessage(#selector(UIViewController.viewWillAppear(_:)))
            .mapToVoid()
            .asDriverOnErrorJustComplete()
        let pull = tableView.refreshControl!.rx
            .controlEvent(.valueChanged)
            .asDriver()
        
        let input = RecipeListVM.Input(trigger: Driver.merge(viewWillAppear, pull),
                                       createPostTrigger: (navigationItem.rightBarButtonItem?.rx.tap.asDriver())!,
                                       selection: tableView.rx.itemSelected.asDriver())
        let output = viewModel?.transform(input: input)
        //Bind Posts to UITableView
        output?.recipes.drive(
            tableView.rx.items(cellIdentifier: "RecipeItemCell", cellType: RecipeItemCell.self)
        ) { (index, recipe: Recipe, cell) in
            
            cell.setup(recipe)
            }.disposed(by: disposeBag)
        //Connect Create Post to UI
        
        output?.fetching
            .drive(tableView.refreshControl!.rx.isRefreshing)
            .disposed(by: disposeBag)
        output?.createPost
            .drive()
            .disposed(by: disposeBag)
        output?.selectedPost
            .drive()
            .disposed(by: disposeBag)
    }
}
