//
//  FirebaseAuthService.swift
//  DontWaste
//
//  Created by David Aldorf on 25.02.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import FirebaseAnalytics
import FirebaseAuth
import FacebookCore
import FacebookLogin
import RxSwift

enum FirebaseAuthError: Error {
    case failed
}

class FirebaseAuthService {
    
    func signIn(email: String, password: String) -> Observable<User?> {
        
        return Auth.auth().rx_signinWithEmail(email: email, password: password)
    }
    
    func signInWithFacebook() -> Observable<User?>{
        
        guard let accessToken = AccessToken.current?.authenticationToken else {
            
            return Observable.create { observer in
                
                observer.onError(FirebaseAuthError.failed)
                
                return Disposables.create()
            }
        }
        
        let credential = FacebookAuthProvider.credential(withAccessToken: accessToken)
        
        return Auth.auth().rx_signInWithCredentials(credentials: credential)
    }
    
    func createUser(email: String, password: String) -> Observable<User?> {
        
        return Auth.auth().rx_createUserWithEmail(email: email, password: password)
    }
    
    var isUserAuthenticated: Bool {
        
        return true
    }
}
