//
//  Don_t_WasteTests.swift
//  Don't WasteTests
//
//  Created by David Aldorf on 24.02.18.
//  Copyright © 2018 David Aldorf. All rights reserved.
//

import XCTest
import Firebase
import RxSwift
import RxCocoa
import RxBlocking
@testable import DontWaste

enum TestError: Error {
    case test
}

class PostsViewModelTests: XCTestCase {
    
var mockedFirebaseService: MockFirebaseService!
var mockedNavigator: MockedNavigator!
var viewModel: FridgeListVM!
let disposeBag = DisposeBag!

override func setUp() {
    super.setUp()
    mockedFirebaseService = MockFirebaseService()
    mockedNavigator = MockedNavigator()
    disposeBag = DisposeBag()
    viewModel = FridgeListVM(useCase: mockedFirebaseService,
                               navigator: mockedNavigator)
}

    func test_transform_triggerInvoked_mapProductsToViewModels() {
        // arrange
        let trigger = PublishSubject<Void>()
        let output = viewModel.transform(input: createInput(trigger: trigger))
        allPostUseCase.posts_ReturnValue = Observable.just(createPosts())
        
        // act
        output.posts.drive().disposed(by: disposeBag)
        trigger.onNext()
        let posts = try! output.posts.toBlocking().first()!
        
        // assert
        XCTAssertEqual(posts.count, 2)
    }
    
private func createInput(
    trigger: Observable<Void> = Observable.just(),
    createPostTrigger: Observable<Void> = Observable.never(),
    selection: Observable<IndexPath> = Observable.never()
    ) -> FridgeListVM.Input {

    return FridgeListVM.Input(
        trigger: trigger.asDriverOnErrorJustComplete(),
        createPostTrigger: createPostTrigger.asDriverOnErrorJustComplete(),
        selection: selection.asDriverOnErrorJustComplete())
}
    
    private func createPosts() -> [Post] {
        return [
            Post(body: "body 1", title: "title 1", uid: "uid 1", userId: "userId 1"),
            Post(body: "body 2", title: "title 2", uid: "uid 2", userId: "userId 2")
        ]
    }
}

class Don_t_WasteTests: XCTestCase {
    
    override func setUp() {
        super.setUp()
        // Put setup code here. This method is called before the invocation of each test method in the class.
    }
    
    override func tearDown() {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
        super.tearDown()
    }
    
    func testExample() {
        // This is an example of a functional test case.
        // Use XCTAssert and related functions to verify your tests produce the correct results.
    }
    
    func testPerformanceExample() {
        // This is an example of a performance test case.
        self.measure {
            // Put the code you want to measure the time of here.
        }
    }
    
}
